@@ -57,6 +57,9 @@ class VariableError(Exception):
 class PasswordError(VariableError):
     """Exception of password change error"""
 
+class CommonVariableError(VariableError):
+    """Exception for command line"""
+
 import cl_template
 
 def makePath(dirs,mode=755):
@@ -1285,17 +1288,20 @@ class DataVars(SimpleDataVars):
         else:
             filterFunc = lambda x:x
 
+        if not type(where) in (tuple,list):
+            where = [where]
+        woffset = len(where)
         if type(selField) in (tuple,list):
-            count = len(selField)
-            mapFunc = lambda x:x[1:]
+            count = len(selField)+woffset
+            mapFunc = lambda x:x[woffset:]
             res = filter(filterFunc,
-                  zipVars(where,*selField))
+                  zipVars(*(where+selField)))
         else:
-            count = 1
-            fields = [where,selField]
-            mapFunc = lambda x:x[1]
+            count = 1+woffset
+            fields = where+[selField]
+            mapFunc = lambda x:x[woffset]
             res = filter(filterFunc,
-                  zipVars(where,selField))
+                  zipVars(*fields))
         if sort:
             if "/" in sort:
                 sort,sortkey = sort.split('/')
@@ -1303,7 +1309,7 @@ class DataVars(SimpleDataVars):
             res.sort(key=sortkey,reverse=True if sort == "DESC" else False)
         if limit == 1:
             if not any(res):
-                res = [[""]*(count+1)]
+                res = [[""]*(count)]
             return mapFunc(res[0])
         else:
             return map(mapFunc,res[:limit])
@@ -1327,9 +1333,22 @@ class DataVars(SimpleDataVars):
                 varObj.value.close()
 
     def addGroup(self,groupName,normal=(),expert=(),brief=(),next_label=None,
-                 hide=(), expert_label=None,image=''):
+                 brief_force=(), hide=(), expert_label=None,image=''):
         """
         Add group by name, normal, expert and next_label
+
+        name - variable name
+        normal - variable display as normal variable
+        expert - variable display in expert mode
+        hide - hide variable in brief
+        image - image (GUI)
+        brief - variable display in brief if it is compatible (at example
+                hide os_migrate_data and display os_migrate_user for brief
+                display)
+        brief_force - display variable even if it is uncompatible (at example
+                      os_location_data with using autopartition)
+        expert_label - replace standard open expert variables label (GUI)
+        next_label - label on button for next step (GUI)
         """
         default_next_label = _("Next")
         default_expert_label = _("Click for advanced settings")
@@ -1340,6 +1359,7 @@ class DataVars(SimpleDataVars):
              'hide':hide,
              'image':image,
              'brief':brief,
+             'brief_force':brief_force,
              'expert_label':expert_label or default_expert_label,
              'next_label':next_label or default_next_label})
         lastIndex = len(self.groups)-1
@@ -1493,10 +1513,14 @@ class DataVars(SimpleDataVars):
                     messages = e.message if type(e.message) == list \
                                else [e]
                     mess = "\n".join(map(lambda x:str(x),self.plainList(*messages)))
-                    if isinstance(e,PasswordError) or \
-                        type(e.message) == list and \
-                        all(isinstance(x,PasswordError) for x in e.message):
-                        typeError = 'pwderror'
+                    mapError = {PasswordError:'pwderror',
+                                CommonVariableError:'commonerror'}
+                    for k,v in mapError.items():
+                        if isinstance(e,k) or \
+                            type(e.message) == list and \
+                            all(isinstance(x,k) for x in e.message):
+                            typeError = v
+                            break
                     else:
                         typeError = 'error'
                     errors.append({'type':typeError, 'field':var, 'message':mess})
