@@ -83,6 +83,7 @@ class process:
         self.langc = "langc" in kwarg
 
         self.stderr = kwarg.get("stderr",PIPE)
+        self.cwd = kwarg.get("cwd",None)
         self.command = [command] + list(params)
         self.stdin = stdin
         self.iter = None
@@ -98,6 +99,7 @@ class process:
                               stdout=self.stdout,
                               stdin=self.stdin(),
                               stderr=self.stderr,
+                              cwd=self.cwd,
                               env=self.envdict)
 
     def _defaultStdin(self):
@@ -272,7 +274,18 @@ class typeFile:
 
     def getMType(self, filename):
         """Информация о типе файла"""
-        return self.magicObject.file(filename)
+        ret = self.magicObject.file(filename)
+        # fix for kernel 3.7.7 (bad work samba)
+        if ret is None and self.magicObject.errno() == 5:
+            r,w = os.pipe()
+            devnull = os.open(os.devnull,os.O_WRONLY)
+            cat = subprocess.Popen(['/bin/cat',filename],stdout=w,
+                    stderr=devnull,close_fds=True)
+            ret = self.magicObject.descriptor(r)
+            os.close(w)
+            os.close(devnull)
+            cat.wait()
+        return ret
 
     def isBinary(self, filename):
         """является ли файл бинарным"""
@@ -709,12 +722,12 @@ class FStab(object):
     fstab_file = '/etc/fstab'
     NAME, DIR, TYPE, OPTS, FREQ, PASSNO = range(0,6)
 
-    def __init__(self,fstab_file=None):
+    def __init__(self,fstab_file=None,devs=[]):
         if fstab_file:
             self.fstab_file = fstab_file
         self.cache = []
         self.rotateCache = []
-        self.dictUUID = getUUIDDict()
+        self.dictUUID = getUUIDDict(devs=devs)
         self.rebuildCache()
 
     def rebuildCache(self):
