@@ -21,6 +21,7 @@ from calculate.lib import  cl_overriding
 import re
 import sys
 from types import StringType
+from itertools import *
 
 from re import search, compile, S
 from calculate.lib.cl_lang import setLocalTranslate
@@ -136,100 +137,58 @@ def prettyColumnStr(*cols):
         retstr += "\n"
     return retstr.encode('utf8')
 
-def columnStr(*cols):
-    '''Вывод данных по колонкам, причем, если данные не вмещаются в указнаную
-    колонку, то они переносятся на следующую строку в нужную колонку. В строку.
-
-    Параметры:
-    cols     множестово пар: текст, ширина колонки, причем, если у последней
-             колонки не указывать ширину, то она будет выведена вся.
-
-    Возвращаемые параметры:
-    строка, которую можно использовать для вывода на экран
-
-    Пример: columnWrite( "Some text", 10, "Next column", 20 )
-    '''
-    retstr = ""
-    # перевести кортеж в список, т.к. изменяется
-    cols = list(cols)
-    # перевести текст в юникод, заодно перевести числа в строку
-    for i in xrange(0,len(cols),2):
-        cols[i] = (str(cols[i])).decode('utf8')
-
-    # флаг "есть еще текст для вывода"
-    repeat = True
-    while repeat:
-        # сбросить итератор на первый элемент
-        q = 0
-        repeat = False
-        # пока не закончили перебирать параметры (перебираем по парам)
-        while q < len(cols):
-            # если это последний параметр, и для него не указана ширина
-            if q == len(cols)-1:
-                # выводим его полностью не смотря на ширину окна
-                retstr += cols[q] + " "
-                cols[q] = ''
+def columnMatrix(*cols):
+    """
+    Split columns text to matrix
+    """
+    def splitter(s,wid):
+        """Split string by width and \\n"""
+        for part in unicode(s).split('\n'):
+            if wid > 0:
+                while part:
+                    p, part = part[:wid],part[wid:]
+                    yield p
             else:
-                # вывести часть строки не больше указанной ширины колонки
-                retstr+=(cols[q][:cols[q+1]].ljust(cols[q+1])).encode('utf8') \
-                    + " "
-                # остальную часть строки оставить на следующую итерацию
-                cols[q] = cols[q][cols[q+1]:]
-                # если от строки что то осаталось
-                if len(cols[q]) > 0:
-                    # отметить запуск еще одной итерации по параметрам
-                    repeat = True
-            # следующая пара
-            q += 2
-        # колонки отображены
-        retstr += "\n"
-    return retstr
+                yield part
+    return izip_longest(
+           *starmap(splitter,
+           izip_longest(islice(cols,0,None,2),
+                        islice(cols,1,None,2))),
+           fillvalue="")
+
+def columnStr(*cols):
+    '''Get string for write, data by columns
+
+    Parameters:
+    cols     list(tuple(text,width))
+
+    Return:
+    string for display
+
+    Example: columnStr( "Some text", 10, "Next column", 20 )
+    '''
+    
+    def generateStrings(*cols):
+        colWidth = list(islice(cols,1,None,2))
+        for line in map(lambda x:izip_longest(colWidth,x),columnMatrix(*cols)):
+            yield " ".join(("{0:%d}"%(width or 1)).format(s) for \
+                            width,s in line)
+    return "\n".join(generateStrings(*cols))
 
 def columnWrite(*cols):
-    '''Вывод данных по колонкам, причем, если данные не вмещаются в указнаную
-    колонку, то они переносятся на следующую строку в нужную колонку.
+    '''Display data by columns
 
-    Параметры:
-    cols     множестово пар: текст, ширина колонки, причем, если у последней
-             колонки не указывать ширину, то она будет выведена вся.
+    Parameters:
+    cols     list(tuple(text,width))
 
-    Пример: columnWrite( "Some text", 10, "Next column", 20 )
+    Example: columnWrite( "Some text", 10, "Next column", 20 )
     '''
-    # перевести кортеж в список, т.к. изменяется
-    cols = list(cols)
-    # перевести текст в юникод, заодно перевести числа в строку
-    for i in xrange(0,len(cols),2):
-        cols[i] = (str(cols[i])).decode('utf8')
-
-    # флаг "есть еще текст для вывода"
-    repeat = True
-    while repeat:
-        # сбросить итератор на первый элемент
-        q = 0
-        repeat = False
-        # пока не закончили перебирать параметры (перебираем по парам)
-        while q < len(cols):
-            # если это последний параметр, и для него не указана ширина
-            if q == len(cols)-1:
-                # выводим его полностью несмотря на ширину окна
-                cl_overriding.printSUCCESS(cols[q].encode('utf8'),
-                                           printBR=False)
-                cols[q] = ''
-            else:
-                # вывести часть строки не больше указанной ширины колонки
-                cl_overriding.printSUCCESS(\
-                    (cols[q][:cols[q+1]].ljust(cols[q+1])).encode('utf8') + " ",
-                    printBR=False)
-                # остальную часть строки оставить на следующую итерацию
-                cols[q] = cols[q][cols[q+1]:]
-                # если от строки что то осаталось
-                if len(cols[q]) > 0:
-                    # отметить запуск еще одной итерации по параметрам
-                    repeat = True
-            # следующая пара
-            q += 2
-        # колонки отображены
-        cl_overriding.printSUCCESS('')
+    colWidth = list(islice(cols,1,None,2))
+    for line in map(lambda x:izip_longest(colWidth,x),columnMatrix(*cols)):
+        for width,s in line:
+            cl_overriding.printSUCCESS(("{0:%d} "%(width or 1)).format(s),
+                printBR=False)
+        cl_overriding.printSUCCESS("",printBR=True)
 
 def justify(s,width):
     '''Выровнить текст по ширине
@@ -266,16 +225,43 @@ def justify(s,width):
     return s.encode('utf-8')
 
 class tableReport:
-    """Класс для выдачи данных в табличном виде"""
-    def __init__(self, title, headerList, dataList):
-        # Заголовок
+    """Display data in table"""
+    def __init__(self, title, headerList, dataList,colSpan=1):
+
+        # title
         self.title = title
-        # Cписок элементов первой строки таблицы
+        # elements list for first table row
         self.headerList = headerList
-        # Список строк (каждая строка - список элементов)
-        self.dataList = dataList
-        # Список ширин колонок
+        # row list (each row is list)
+        self.dataList = map(lambda x:map(lambda y:str(y).expandtabs(),
+                                     map(lambda y:y or "",x)),
+                        dataList)
+        # calculate columns list
         self.columnsWidth = self.getColumsnWidth()
+        self.vertChar = colSpan*" "+"|"+colSpan*" "
+        self.crossChar = colSpan*"-"+"+"+colSpan*"-"
+        self.colSpan = colSpan
+        self.align = "<"+"^"*(len(headerList)-2)+"<"
+
+    def setAutosize(self,colNum=-1,maxSize=None,forceMax=False):
+        """
+        Set width for specified column
+        """
+        if maxSize is None:
+            maxSize = getTerminalSize()[1]
+            if not maxSize:
+                maxSize = 80
+        spanSize = self.colSpan*2+1
+        if colNum < 0:
+            colNum = len(self.columnsWidth)+colNum
+        currentSize = sum(map(lambda x:x+spanSize,self.columnsWidth))+1
+        if currentSize < maxSize and not forceMax:
+            return
+        excludeSize = sum(map(lambda x:x[1]+spanSize,
+                      filter(lambda x:x[0]!=colNum,
+                      enumerate(self.columnsWidth))))+spanSize+1
+        if maxSize - excludeSize > 5:
+            self.columnsWidth[colNum] = maxSize - excludeSize
 
     def getColumsnWidth(self):
         """Находит максимальную ширину каждой колонки
@@ -310,63 +296,43 @@ class tableReport:
 
     def createFormatStr(self, listStr):
         """Создает список (текст, ширина ...)"""
-        strList = []
-        lenStr = len(listStr)
-        for i in range(len(self.columnsWidth)):
-            if lenStr > i:
-                strList.append(listStr[i])
-            else:
-                strList.append("")
-            strList.append(self.columnsWidth[i])
-        return strList
+        return chain(*izip_longest(listStr,self.columnsWidth,fillvalue=""))
+
+
+    def printFunc(self,s):
+        """
+        Overriding print
+        """
+        cl_overriding.printSUCCESS(s)
 
     def printReport(self,printRows=True):
         """Напечатать данные в табличном виде"""
         if self.title:
-            cl_overriding.printSUCCESS(self.title)
+            self.printFunc(self.title)
         listStrSep = []
         for lenCol in self.columnsWidth:
             listStrSep.append("-"*lenCol)
-        printData = ""
-        printData += columnStr(*self.createFormatStr(listStrSep))
-        printData += columnStr(*self.createFormatStr(self.headerList))
-        printData += columnStr(*self.createFormatStr(listStrSep))
+        printData = []
+        printData.extend(columnMatrix(*self.createFormatStr(listStrSep)))
+        printData.extend(columnMatrix(*self.createFormatStr(self.headerList)))
+        printData.extend(columnMatrix(*self.createFormatStr(listStrSep)))
         for s in self.dataList:
-            printData += columnStr(*self.createFormatStr(s))
-        printData += columnStr(*self.createFormatStr(listStrSep))
-        if printData[-1] == "\n":
-            printData = printData[:-1]
-        lines = printData.splitlines()
-        lenCols = map(lambda x: len(x), lines[0].strip().split(" "))
-        convLines = []
-        lenLines = len(lines)
-        for i in range(lenLines):
-            char = " | "
+            printData.extend(columnMatrix(*self.createFormatStr(s)))
+        printData.extend(columnMatrix(*self.createFormatStr(listStrSep)))
+        lenLines = len(printData)
+        for i,line in enumerate(
+                      map(lambda x:zip(self.align,self.columnsWidth,x),
+                      printData)):
+            char = self.vertChar
             if i == 0 or i == 2 or i == lenLines-1:
-                char ="-+-"
-            convLines.append(self._insertStrChar(lines[i], lenCols, char))
-        cl_overriding.printSUCCESS("\n".join(convLines))
+                char = self.crossChar
+            self.printFunc(
+                "%s%s%s"%(char[self.colSpan:],
+                char.join(starmap(lambda align,width,txt:("{0:%s%d}"%
+                                  (align,width or 1)).format(txt),line)),
+                char[:len(char)-self.colSpan]))
         if printRows:
-            cl_overriding.printSUCCESS("(%s %s)"%(len(self.dataList), _("rows")))
-
-    def _insertStrChar(self, line, lenCols, char):
-        """Вставляет несколько символов char в указанные позиции
-
-        сначала строки будет вставлено char[1:] в конце строки char[:-1]
-        """
-        lineUnicode = _toUNICODE(line)
-        prevPos = 0
-        convLine = char[1:]
-        lenLenCols = len(lenCols)
-        for i in range(lenLenCols):
-            pos = lenCols[i] + prevPos
-            if i == lenLenCols-1:
-                insertChar = char[:-1]
-            else:
-                insertChar = char
-            convLine += lineUnicode[prevPos:pos] + insertChar
-            prevPos = pos + 1
-        return convLine.encode("UTF-8")
+            self.printFunc("(%s %s)"%(len(self.dataList), _("rows")))
 
 class MultiReplace:
     """MultiReplace function object
@@ -499,4 +465,24 @@ def formatListOr(lst):
     """Convert list to string like this: [1,2,3] -> 1,2 or 3"""
     lststr = map(lambda x:str(x),lst)
     return (" %s "%_("or")).join(filter(lambda x:x,
-                                 [",".join(lststr[:-1]),"".join(lststr[-1:])]))
+                                 [",".join(map(lambda x:x or "''",lststr[:-1])),
+                                 "".join(lststr[-1:])]))
+
+def getTerminalSize():
+    """
+    Get terminal size
+    """
+    import struct
+    try:
+        import fcntl
+        import termios
+    except ImportError:
+        return (None, None)
+
+    try:
+        data = fcntl.ioctl(sys.stdout.fileno(), termios.TIOCGWINSZ, 4 * '00')
+        height, width = struct.unpack('4H', data)[:2]
+        return (height, width)
+    except Exception:
+        return (None, None)
+
